/*
** Module   : AccessorWrapper.java
** Abstract : wraps a BaseDataType object in an Accessor interface
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --JPRM-- ------------------Description-------------------
** 001 GES 20060204   @24275 Created initial version.  This is an API
**                           for an object that can provide access
**                           (get/set) to a BaseDataType.
** 002 CA  20130112          When the wrapped data is a handle, get() will
**                           convert it to its numeric representation and
**                           set() will change it back to a handle, using the
**                           handle.fromString call. This is needed to be able
**                           to display the handle's hashCode to terminal, as
**                           handle instances can't be serialized and sent to
**                           the client.
*/

package com.goldencode.p2j.util;

/**
 * Defines methods an object that can provide access (get/set) to a
 * {@link BaseDataType}.  This provides a means of delegating the
 * access to an object, such that it can be invoked at an appropriate time
 * by code that does not have direct access or knowledge of the original
 * <code>BaseDataType</code> instance.
 */
public class AccessorWrapper
implements Accessor
{
   /** The wrappered data. */
   private BaseDataType wrappee = null;
   
   /**
    * Construct an instance that references the given wrappee.
    *
    * @param    wrappee
    *           The object to wrapper.  Must NOT be <code>null</code>.
    */
   public AccessorWrapper(BaseDataType wrappee)
   {
      this.wrappee = wrappee;
   }
   
   /**
    * Get a copy of the referenced object's current runtime value (this does
    * not return the original instance itself).
    * <p>
    * In case of handles, an {@link integer} value is returned, which holds
    * the handle's numeric representation.
    *
    * @return   The copy of the current value.
    */
   public BaseDataType get()
   {
      if (wrappee instanceof handle)
      {
         return new integer(wrappee.hashCode());
      }
      return wrappee.duplicate();
   }
   
   /**
    * Set the referenced object's current runtime value to the given value.
    * <p>
    * In case of handles, the <code>val</code> must be its numeric 
    * representation, which is used to create the handle back using the 
    * {@link handle#fromString(String)} call.
    *
    * @param    val
    *           The current value to be assigned into the referenced object.
    */
   public void set(BaseDataType val)
   {
      if (wrappee instanceof handle)
      {
         wrappee = handle.fromString(val.toStringMessage());
      }
      else
      {
         wrappee.assign((Undoable) val);
      }
   }
}
