/*
** Module   : ExternalProgramWrapper.java
** Abstract : container that implements persistent-procedure related APIs.
**
** Copyright (c) 2012, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- -----------------------Description------------------------
** 002 CA  20121220 Created initial version.
*/

package com.goldencode.p2j.util;

import com.goldencode.p2j.ui.LogicalTerminal;
import com.goldencode.p2j.ui.WindowWidget;
import com.goldencode.p2j.util.ProcedureManager.SearchMode;

/**
 * This container adds support for all persistent-procedure related attributes
 * and methods, such as:
 * <ul>
 *   <li>INTERNAL-ENTRIES</li>
 *   <li>NAME</li>
 *   <li>FILE-NAME</li>
 *   <li>PERSISTENT</li>
 *   <li>TYPE</li>
 *   <li>NEXT-SIBLING</li>
 *   <li>PREV-SIBLING</li>
 *   <li>PRIVATE-DATA</li>
 *   <li>GET-SIGNATURE</li>
 *   <li>ADD-SUPER-PROCEDURE</li>
 *   <li>REMOVE-SUPER-PROCEDURE</li>
 *   <li>SUPER-PROCEDURES</li>
 * </ul>
 */
public class ExternalProgramWrapper
extends TransparentWrapper
implements PersistentProcedure
{
   /**
    * Constructs a new wrapper for the given external procedure instance.
    *  
    * @param    referent
    *           The external procedure instance.
    */
   public ExternalProgramWrapper(Object referent) {
      super(referent);
   }

   public boolean isValid()
   {
      if (!super.isValid())
      {
         return false;
      }
      
      return ProcedureManager.hasReferent(referent);
   }
   
   /**
    * API needed to implement read-only attribute assignment (a 4GL 
    * "feature").
    * 
    * @param    attribute
    *           The attribute's name.
    */
   public void readOnlyError(String attribute)
   {
      handle.readOnlyError(new handle(this), attribute);
   }
   
   /**
    * Get the <code>INTERNAL-ENTRIES</code> attribute of this 
    * {@link TransparentWrapper#referent external procedure instance.}
    * 
    * @return   See above.
    */
   public character internalEntries()
   {
      String pname = getFileName().toStringMessage();
      boolean prv = this.referent == ProcedureManager.thisProcedure().get();

      return new character(SourceNameMapper.getInternalEntries(pname, prv));
   }

   /**
    * Get the <code>NAME</code> attribute of this 
    * {@link TransparentWrapper#referent external procedure instance.}
    * 
    * @return   See above.
    */
   public character getName()
   {
      return getFileName();
   }

   /**
    * Get the <code>FILE-NAME</code> attribute of this 
    * {@link TransparentWrapper#referent external procedure instance.}
    * 
    * @return   See above.
    */
   public character getFileName()
   {
      return ProcedureManager.getFileName(referent);
   }

   /**
    * Get the <code>persistent</code> attribute of this 
    * {@link TransparentWrapper#referent external procedure instance.}
    * 
    * @return   See above.
    */
   public logical isPersistent()
   {
      return ProcedureManager.isPersistent(referent);
   }
   
   /**
    * Get the <code>TYPE</code> attribute of this 
    * {@link TransparentWrapper#referent external procedure instance.}
    * 
    * @return   Always returns <code>"PROCEDURE"</code>.
    */
   public character getType()
   {
      // TODO: add support for other types of external programs
      return new character("PROCEDURE");
   }

   /**
    * Get the <code>NEXT-SIBLING</code> attribute of this 
    * {@link TransparentWrapper#referent external procedure instance.}
    * 
    * @return   See above.
    */
   public handle getNextSibling()
   {
      return ProcedureManager.getNextSibling(referent);
   }

   /**
    * Get the <code>PREV-SIBLING</code> attribute of this 
    * {@link TransparentWrapper#referent external procedure instance.}
    * 
    * @return   See above.
    */
   public handle getPrevSibling()
   {
      return ProcedureManager.getPrevSibling(referent);
   }

   /**
    * Get the <code>PRIVATE-DATA</code> attribute of this 
    * {@link TransparentWrapper#referent external procedure instance.}
    * 
    * @return   See above.
    */
   public character getPrivateData()
   {
      return ProcedureManager.getPrivateData(referent);
   }

   /**
    * Set the <code>PRIVATE-DATA</code> attribute of this 
    * {@link TransparentWrapper#referent external procedure instance.}
    * 
    * @param    data
    *           The new private-data value. 
    */
   public void setPrivateData(character data)
   {
      ProcedureManager.setPrivateData(referent, data);
   }

   /**
    * Set the <code>PRIVATE-DATA</code> attribute of this 
    * {@link TransparentWrapper#referent external procedure instance.}
    * 
    * @param    data
    *           The new private-data value. 
    */
   public void setPrivateData(String data)
   {
      setPrivateData(new character(data));
   }

   /**
    * Execute the <code>GET-SIGNATURE</code> method of this 
    * {@link TransparentWrapper#referent external procedure instance.}
    * 
    * @param    internalEntry
    *           The legacy name of the internal-entry for which the signature
    *           is needed. 
    */
   public character getSignature(String internalEntry)
   {
      return getSignature(new character(internalEntry));
   }

   /**
    * Execute the <code>GET-SIGNATURE</code> method of this 
    * {@link TransparentWrapper#referent external procedure instance.}
    * 
    * @param    internalEntry
    *           The legacy name of the internal-entry for which the signature
    *           is needed. 
    */
   public character getSignature(character internalEntry)
   {
      String ieName = internalEntry.toStringMessage();
      character fileName = getFileName();
      String pname = fileName.toStringMessage();

      if ("".equals(ieName))
      {
         return new character(SourceNameMapper.getSignature(pname, ""));
      }

      // is safe to call both, because: if there are both func and proc with
      // the same name, then the signature is for the func, not the proc
      // (thus if the func is private, then is not seen).
      // else, if no func with this name, proc is cheched
      if (SourceNameMapper.isPrivate(pname, ieName, true) ||
          SourceNameMapper.isPrivate(pname, ieName, false))
      {
         // am I calling this from THIS-PROCEDURE? if yes, then I have access
         handle thisp = ProcedureManager.thisProcedure();
         character thisName = handle.unwrapProcedure(thisp)
                                    .getFileName();
         if (!CompareOps.equals(fileName, thisName))
         {
            // TODO: what is the correct value ?
            return new character();
         }
      }

      return new character(SourceNameMapper.getSignature(pname, ieName));
   }

   /**
    * Get the <code>SUPER-PROCEDURES</code> attribute of this 
    * {@link TransparentWrapper#referent external procedure instance.}
    * 
    * @return   See above.
    */
   public character superProcedures()
   {
      return ProcedureManager.superProcedures(referent);
   }

   /**
    * Add the given handle as a super-procedure for the current handle.
    * 
    * @param    h
    *           The handle to be added as a super-procedure.
    *           
    * @return   <code>true</code> if <code>h</code> is a valid procedure
    *           handle.
    *           
    * @throws   IllegalArgumentException    
    *           If <code>h</code> is not a {@link handle} instance.
    */
   public logical addSuperProcedure(BaseDataType h)
   {
      return addSuperProcedure(handle.getHandle(h));
   }

   /**
    * Add the given handle as a super-procedure for the current handle, using
    * the explicit {@link SearchMode mode}.
    * 
    * @param    h
    *           The handle to be added as a super-procedure.
    * @param    mode
    *           The {@link SearchMode} to be set for the <code>h</code> 
    *           handle.
    *           
    * @return   <code>true</code> if <code>h</code> is a valid procedure
    *           handle.
    *           
    * @throws   IllegalArgumentException    
    *           If <code>h</code> is not a {@link handle} instance.
    */
   public logical addSuperProcedure(BaseDataType h, SearchMode mode)
   {
      return addSuperProcedure(handle.getHandle(h), mode);
   }
   
   /**
    * Add the given handle as a super-procedure for the current handle, using
    * the search-mode set as a string (one of the <code>SEARCH-TARGET</code>
    * or <code>SEARCH-SELF</code> string literals).
    * 
    * @param    h
    *           The handle to be added as a super-procedure.
    * @param    mode
    *           The search-mode to be set for the <code>h</code> handle.
    *           
    * @return   <code>true</code> if <code>h</code> is a valid procedure
    *           handle.
    *           
    * @throws   IllegalArgumentException    
    *           If <code>h</code> is not a {@link handle} instance.
    */
   public logical addSuperProcedure(BaseDataType h, String mode)
   {
      return addSuperProcedure(handle.getHandle(h), mode);
   }

   /**
    * Add the given handle as a super-procedure for the current handle.
    * 
    * @param    h
    *           The handle to be added as a super-procedure.
    *           
    * @return   <code>true</code> if <code>h</code> is a valid procedure
    *           handle.
    */
   public logical addSuperProcedure(handle h)
   {
      return ProcedureManager.addSuperProcedure(referent, h);
   }

   /**
    * Add the given handle as a super-procedure for the current handle, using
    * the explicit {@link SearchMode mode}.
    * 
    * @param    h
    *           The handle to be added as a super-procedure.
    * @param    mode
    *           The {@link SearchMode} to be set for the <code>h</code> 
    *           handle.
    *           
    * @return   <code>true</code> if <code>h</code> is a valid procedure
    *           handle.
    */
   public logical addSuperProcedure(handle h, SearchMode mode)
   {
      return ProcedureManager.addSuperProcedure(referent, h, mode);
   }

   /**
    * Add the given handle as a super-procedure for the current handle, using
    * the search-mode set as a string (one of the <code>SEARCH-TARGET</code>
    * or <code>SEARCH-SELF</code> string literals).
    * 
    * @param    h
    *           The handle to be added as a super-procedure.
    * @param    mode
    *           The search-mode to be set for the <code>h</code> handle.
    *           
    * @return   <code>true</code> if <code>h</code> is a valid procedure
    *           handle.
    */
   public logical addSuperProcedure(handle h, String mode)
   {
      return ProcedureManager.addSuperProcedure(referent, h, mode);
   }

   /**
    * Remove the given handle from this handle's list of super-procedures.
    *  
    * @param    h
    *           The procedure handle to be removed.
    *           
    * @return   <code>true</code> if <code>h</code> is a valid procedure
    *           handle.
    *           
    * @throws   IllegalArgumentException    
    *           If <code>h</code> is not a {@link handle} instance.
    */
   public logical removeSuperProcedure(BaseDataType h)
   {
      return removeSuperProcedure(handle.getHandle(h));
   }

   /**
    * Remove the given handle from this handle's list of super-procedures.
    *  
    * @param    h
    *           The procedure handle to be removed.
    *           
    * @return   <code>true</code> if <code>h</code> is a valid procedure
    *           handle.
    */
   public logical removeSuperProcedure(handle h)
   {
      return ProcedureManager.removeSuperProcedure(referent, h);
   }

   /**
    * Get CURRENT-WINDOW attribute for given procedure handle.
    *          
    * @return  Reference to current window widget.
    */
   public WindowWidget currentWindow()
   {
      handle phandle = new handle(this);
      // TODO: handle validation
      return LogicalTerminal.currentWindow(phandle);
   }

   /**
    * Set CURRENT-WINDOW for this handle.
    * 
    * @param   cWin
    *          Window reference.
    */
   public void setProcCurrentWindow(WindowWidget cWin)
   {
      handle phandle = new handle(this);
      // TODO: handle validation
      LogicalTerminal.setProcCurrentWindow(phandle, cWin);
   }
}